#!/usr/bin/env python -B
## part of power installer

import sys
import os, platform
import subprocess
import tarfile
import re
from tempfile import mkstemp
from coreconf import corepath, show

__file__ = sys.argv[1]
i_theme = sys.argv[2]
i_icon = sys.argv[3]
i_plank = sys.argv[4]
_first_run = sys.argv[5]
_notify = sys.argv[6]

_type_ = ''
logdebpath='/tmp/pi_deb_log.txt'

def launch(cmd):
    distname, tmpv, tmpc = platform.dist()

    # quotes must be around distname
    if distname == '"elementary OS"':
        subprocess.call(['pantheon-terminal', '-e', cmd])
    else:
        subprocess.call(['xterm', '-e', cmd])

name, ext = os.path.splitext(__file__)
if ext in ['.gz', '.bz2']:
    ext = os.path.splitext(name)[1] + ext

if ext == '':
    mimetype = subprocess.check_output(['mimetype', '-b', __file__])

    # do not use if mimetype == shellscript: for some reasons it fails
    if 'application/x-shellscript' in mimetype:
        ext = '.sh'
    else:
        show('The extension of the file could not be detected!', 'true')
        sys.exit(1)

# do not show notification if bash or run file
if not (ext == '.sh' or ext == '.run'):
    if _first_run == 'true':
        show('', '')

if i_theme == 'true':
    _type_   = 'system theme'
    exe_path = '/usr/share/themes/'
elif i_icon == 'true':
    _type_   = 'icon theme'
    exe_path = '/usr/share/icons/'
elif i_plank == 'true':
    _type_   = 'plank theme'
    exe_path = '/usr/share/plank/themes/'
    if not os.path.exists(exe_path):
        show('Sorry, but it looks like you are not running elementary OS or either using plank!', 'true')
        sys.exit(1)

def pkgerr():
    show('Something went wrong. The package could be corrupted!', 'true')
    sys.exit(1)


def extract(exe_path, zip=''):
    if ext == '.zip':
        import zipfile
        zip = zipfile.ZipFile(__file__, 'r')
        try:
            zip.extractall(exe_path)
        except IOError:
            pkgerr()
        else:
            show('Successfully installed ' + _type_ + ': ' + __file__ + '!', 'false')
    elif ext == '.tar.gz':
        gz = tarfile.open(__file__, 'r:gz')
        try:
            gz.extractall(exe_path)
        except IOError:
            pkgerr()
        else:
            show('Successfully installed ' + _type_ + ': ' + __file__ + '!', 'false')
        gz.close()
    elif ext == '.tar.bz2':
        bz2 = tarfile.open(__file__, 'r:bz2')
        try:
            bz2.extractall(exe_path)
        except IOError:
            pkgerr()
        else:
            show('Successfully installed ' + _type_ + ': ' + __file__ + '!', 'false')
        bz2.close()
    else:
        show('Something went wrong. Check the extension of the packed archive and try again. The supported formats are: zip, tar.gz and tar.bz2', 'true')

    if zip != '':
        zip.close()
    sys.exit(0)

if _type_ != '':
    extract(exe_path)

# check if supported
regex = re.compile('.deb|.sh|.run|.py|.zip|.tar.gz|.tar.bz2')
if regex.match(ext):
    pass
else:
    show('Sorry, but ' + ext + ' files are not supported by Power Installer!', 'true')
    sys.exit(1)

def debsuccess():
    show('Successfully installed debian file: ' + __file__ + '!', 'false')
    os.remove(logdebpath)

def exedeb():
    logdebfile = open(logdebpath, 'w+r')
    run_deb = subprocess.Popen(['dpkg', '-i', __file__], stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

    # write line to stdout and to file
    for line in run_deb.stdout:
        sys.stdout.write(line)
        logdebfile.write(line)

    streamwait = run_deb.communicate()[0]
    return run_deb.returncode

if os.path.isfile(__file__):
    pass
else:
    show('Sorry, but this file does not exist!', 'true')
    sys.exit(1)

if ext == '.deb':
    outcode = exedeb()
    if outcode == 0:
        debsuccess()
    else:
        print('\nPOWERINSTALLER: Trying to install missing dependencies...\n')
        subprocess.call(['apt-get', 'install', '-y', '-f'])
        outcode = exedeb()
        if outcode == 0:
            debsuccess()
        else:
            show('Something went wrong. Check out: /tmp/pi_deb_log.txt for more information.', 'true')
    sys.exit(0)

if(ext == '.sh' or ext == '.run'):
    if '#![POWER INSTALLER ENTRY]' in open(__file__).read():

        # open file as f
        with open(__file__) as f:

            # read content of the file
            cdata =  f.read()

            fd, export_path = mkstemp(prefix='.', suffix='pi_export_var.sh')
            file_export = open(export_path, 'w+r')

            # bash function for exporting all varaibles
            func_export = '''allkeys="APP_NAME CATEGORY SUBTITLE app_icon website window_title
description version run_command_begin ppa pkg run_command_end debian_link theme_link icon_link plank_link no_display"
if [ "$app_icon" != "" ]; then
    if [ "${app_icon:0:1}" != "/" ]; then
        key_link="true"
        cd "/tmp/"
        wget "$app_icon" -O ".power-installer-custom.png"
        export app_icon="/tmp/.power-installer-custom.png"
        cd
    fi
else
    export app_icon=""
fi
if dpkg -s "$pkg" >/dev/null 2>&1; then
    export status="installed"
else
    export status="not-installed"
fi
for var in $allkeys; do
    if [ "${!var}" = "" ]; then
        export "$var"=""
    else
        export "$var"
    fi
done
killall "power-installer"
INIT_API="true" power-installer
if [ "$key_link" = "true" ]; then
    if [ -f "$app_icon" ]; then
        rm -f "$app_icon"
    fi
fi
'''
            file_export.write(cdata)
            file_export.write(func_export)
            file_export.close()
            subprocess.call(['bash', export_path])

            if os.path.isfile(export_path):
                os.remove(export_path)
    else:
        launch('bash ' + __file__)
        if _notify != 'false':
            show('Successfully executed shell script file: ' + __file__ + '!', 'false')

elif ext == '.py':
    launch('python ' + __file__)
    show('Successfully executed python file: ' + __file__ + '!', 'false')
else:
    show('The file you opened could be an archive and none of the options were checked!', 'true')

sys.exit(0)
