#!/usr/bin/env python -B
## part of power installer

import sys
import os
import subprocess
import apt
from shutil import rmtree
from tempfile import mkstemp, mkdtemp
from coreconf import *

ppa = sys.argv[1]
pkg = sys.argv[2]
cinput = sys.argv[3]
link = sys.argv[4]
opt = sys.argv[5]
installed = sys.argv[6]

def ppa_get_code(sin_ppa, opt = None):
    if opt != None:
        call_cmd = subprocess.Popen(['add-apt-repository', '-y', opt, sin_ppa])
    else:
        call_cmd = subprocess.Popen(['add-apt-repository', '-y', sin_ppa])

    streamwaitppa = call_cmd.communicate()[0]
    cmdoutcode = call_cmd.returncode
    return cmdoutcode

def ppa_error(sin_ppa):
    show('Something went wrong. Check the syntax of repostiory: ' + sin_ppa + ' and try again.', True)

def pkg_error(sin_pkg):
    show('Something went wrong. Check names of the packages and try again.', 'true')

def call_dragdrop(fullpath, theme='false', icon='false', plank='false'):
    subprocess.call(['python', corepath + '/dragdrop.py', fullpath, theme, icon, plank, 'false', 'false'])

def ppa_pkg(ppa, pkg):
    ppasucess = None
    pkgsucess = None
    if ppa != '':
        if installed == 'false':
            print('\nPOWERINSTALLER: Adding repositories...\n')
            methodppa = 'added'
            for sin_ppa in ppa.split():
                ppaoutcode = ppa_get_code(sin_ppa)
                if ppaoutcode == 1:
                    ppa_error(sin_ppa)
        else:
            print('\nPOWERINSTALLER: Removing repositories...\n')
            methodppa = 'removed'
            for sin_ppa in ppa.split():
                ppaoutcode = ppa_get_code(sin_ppa, '-r')
                if ppaoutcode == 1:
                    ppa_error(sin_ppa)

        print('\nPOWERINSTALLER: Updating repositories..\n')
        cacheppa = apt.Cache()
        try:
            cacheppa.update()
        except IOError:
            ppasucess = False
        else:
            ppasucess = True

        if ppasucess == True:
            show('Succesfully ' + methodppa + " PPA's: " + ppa + '!', 'false')
        else:
            show('Something went wrong. The repositories could not be updated properly!', 'true')

    if pkg != '':
        cachepkg = apt.cache.Cache()
        if installed == 'false':
            print('\nPOWERINSTALLER: Installing packages...\n')
            uninstallpkg = False
            methodpkg = 'installed'
            for sin_pkg in pkg.split():
                try:
                    p = cachepkg[sin_pkg]
                except KeyError:
                    pkg_error()
                else:
                    p.mark_install()
                    try:
                        cachepkg.commit()
                    except IOError:
                        pkgsucess = False
                    else:
                        pkgsucess = True
        else:
            for sin_pkg in pkg.split():
                print('\nPOWERINSTALLER: Uninstalling packages...\n')
                methodpkg = 'removed'
                try:
                    p = cachepkg[sin_pkg]
                except KeyError:
                    pkg_error()
                else:
                    p.mark_delete()
                    try:
                        cachepkg.commit()
                    except IOError:
                        pkgsucess = True
                        uninstallpkg = True
                    else:
                        pkgsucess = True
                        uninstallpkg = False

        if pkgsucess == True:
            if uninstallpkg == False:
                show('Succesfully ' + methodpkg + ' packages: ' + pkg + '!', 'false')
            else:
                show('Something went wrong. The package ' + sin_pkg + ' could not be uninstalled!', 'true')
        else:
            show('Something went wrong. The dependencies of the package could not be installed!', 'true')

def exe_cmd(cinput):
    if cinput != '':
        if link == 'false':
            fd, input_path = mkstemp(prefix='.', suffix='_pi_input_cmd.sh')
            input_file = open(input_path, 'w+r')

            # input data
            input_data = '''#!/bin/bash
# AUTOGENERATED BY POWER INSTALLER
cmd() {
    export TERM=xterm
    export USER="''' + user + '"' + '''
    export HOME="''' + homedir + '"\n' + 'cd "$HOME"\n' + cinput + '''
}
(set -e; while true; do echo "y"; sleep 2; done) | cmd
exit "$?"'''
            input_file.write(input_data)
            input_file.close()

            logcmdpath = '/tmp/pi_cmd_log.txt'
            logcmdfile = open(logcmdpath, 'w+r')
            run_cmd = subprocess.Popen(['bash', input_path], stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

            # write line to stdout and to file
            for line in run_cmd.stdout:
                sys.stdout.write(line)
                logcmdfile.write(line)

            run_cmd.communicate()[0]
            out = run_cmd.returncode

            if out == 0:
                show('All commands have been executed successfully!', 'false')
                os.remove(logcmdpath)

            else:
                show('Something went wrong. Check out: /tmp/pi_cmd_log.txt for more information.', 'true')

            if os.path.isfile(input_path):
                os.remove(input_path)
        else:
            tmpname = mkdtemp(prefix='.', suffix='_pi_downloaded')
            os.chdir(tmpname)

            # urllib cannot handle not http websites so we use standard wget
            subprocess.call(['wget', cinput])
            fullpath = tmpname + '/' + os.path.basename(cinput)
            if not os.path.isfile(fullpath):
                show('Something went wrong. The file could not be downloaded, check if your link is vaild!', 'true')
            else:
                if(opt == 'a' or opt == 'b'):
                    call_dragdrop(fullpath)
                elif opt == 'c':
                    call_dragdrop(fullpath, theme='true')
                elif opt == 'd':
                    call_dragdrop(fullpath, icon='true')
                elif opt == 'e':
                    call_dragdrop(fullpath, plank='true')

            # remove temporary directory
            rmtree(tmpname)

if ppa + pkg + cinput != '':
    print("\nPOWERINSTALLER: USER SESSION OPENED\n")
    ppa_pkg(ppa, pkg)
    exe_cmd(cinput)
    print("\nPOWERINSTALLER: USER SESSION CLOSED\n")

sys.exit(0)
