//* -*- Mode: vala; indent-tabs-mode: nil; tab-width: 4 -*-
/* Copyright (c) 2014 Adam Bieńkowski (http://launchpad.net/power-installer)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authored by: Corentin Noel <tintou@mailoo.org>
 */

// commands tab

using Gtk;

namespace PowerInstaller {

    public class CommandTab : Gtk.EventBox {
        private Gtk.TextView    view;
        private Gtk.Entry       ppa_entry;
        private Gtk.Entry       pkg_entry;

        private Gtk.CheckButton link_button;

        private string? user_input;

        private string? ppa;
        private string? pkg;

        private bool    link;

        public CommandTab() {
            // main boxes
            var box = new Gtk.Box(Gtk.Orientation.VERTICAL, 0);
            var ppa_box = new Gtk.Box(Gtk.Orientation.HORIZONTAL, 0);
            var pkg_box = new Gtk.Box(Gtk.Orientation.HORIZONTAL, 0);

            // ppa entry
            ppa_entry = new Gtk.Entry();
            var ppa_label = new Gtk.Label.with_mnemonic("_PPA's to add:    ");
            ppa_label.mnemonic_widget = ppa_entry;
            ppa_entry.text = "";
            ppa_box.add(ppa_label);
            ppa_box.add(ppa_entry);

            // pkg entry
            pkg_entry = new Gtk.Entry();
            pkg_entry.text = "";

            var pkg_label = new Gtk.Label.with_mnemonic("_Packages to install:    ");
            pkg_label.mnemonic_widget = pkg_entry;

            pkg_box.add(pkg_label);
            pkg_box.add(pkg_entry);

            // checkbutton
            link_button = new Gtk.CheckButton.with_label("Is link to debian, shell or python script file");
            link_button.toggled.connect(() => { link = set_link_status(); });

            // textview
            view = new Gtk.TextView();
            view.set_wrap_mode(Gtk.WrapMode.WORD);
            view.buffer.text = "";

            // go button
            var go_button = new Gtk.Button.with_label("Go!");

            // using DESTRUCTIVE_ACTION: more consistent interface
            go_button.get_style_context().add_class("destructive-action");

            go_button.clicked.connect(() => {
                user_input = view.buffer.text;
                ppa = ppa_entry.get_text();
                pkg = pkg_entry.get_text();

                begin_show();
                on_go_clicked(user_input, ppa, pkg, link, 'a', false);
            });

            // add scroll to Gtk.TextView
            var scrolled = new Gtk.ScrolledWindow(null, null);
		    box.pack_start(scrolled, true, true, 0);
		    scrolled.add(view);

            // add everything to box
            box.add(scrolled);
            box.add(link_button);
            box.add(ppa_box);
            box.add(pkg_box);
            box.pack_end(go_button, false, false, 0);
            this.add(box);
        }

        // if link checked
        private bool set_link_status() {
            if(link_button.get_active()) {
                return true;
            } else {
                return false;
            }
        }

        // go has been clicked
        public void on_go_clicked(string user_input, string ppa, string pkg, bool link, char opt, bool installed) {
            execute_command_async(@"python $DATADIR/input.py '$ppa' '$pkg' '$user_input' '$link' '$opt' '$installed'");
        }
    }
}
