//* -*- Mode: vala; indent-tabs-mode: nil; tab-width: 4 -*-
/* Copyright (c) 2014 Adam Bieńkowski (http://launchpad.net/power-installer)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authored by: Corentin Noel <tintou@mailoo.org>
 */

// general tab

using GLib;
using Gdk;
using Gtk;

namespace PowerInstaller {

    public class GeneralTab : Gtk.EventBox {
        private const Gtk.TargetEntry[] targets = {{"text/uri-list", 0, 0}};

        private Gtk.CheckButton icon_button;
        private Gtk.CheckButton theme_button;
        private Gtk.CheckButton plank_button;

        private string file;

        private bool   theme;
        private bool   icon;
        private bool   plank;

        public GeneralTab() {
            create_layout();
            Gtk.drag_dest_set(this, Gtk.DestDefaults.ALL, targets, Gdk.DragAction.COPY);
            drag_data_received.connect(on_drag_data_received);
        }

        private void create_layout() {
            string frame_img = @"$DATADIR/frame.svg";

            // main grid
            var grid = new Gtk.Grid();

            // drag label
            var drag_label = new Gtk.Label("<span size='large'><b> Drag and drop: debian files, shell, python
  scripts, and packed themes here:</b></span>");
            drag_label.set_line_wrap(true);
            drag_label.set_use_markup(true);
            drag_label.set_justify(Gtk.Justification.CENTER);
            drag_label.set_selectable(true);

            // checkbuttons
            theme_button = new Gtk.CheckButton.with_label("Is GTK theme (archived)");
            theme_button.toggled.connect(() => { theme = set_theme_status(); });

            icon_button = new Gtk.CheckButton.with_label("Is icon theme (archived)");
            icon_button.toggled.connect(()  => { icon = set_icon_status(); });

            plank_button = new Gtk.CheckButton.with_label("Is plank theme (elementary OS; archived)");
            plank_button.toggled.connect(() => { plank = set_plank_status(); });

            //frame
            var frame = new Gtk.Image();
            frame.set_from_file(frame_img);

            // add everything to grid
            grid.add(drag_label);
            grid.attach(icon_button, 0, 2, 1, 1);
            grid.attach(theme_button, 0, 1, 1, 1);
            grid.attach(plank_button, 0, 3, 2, 2);
            grid.attach(frame, 0, 5, 3, 3);
            this.add(grid);
        }

        // if theme is checked
        private bool set_theme_status() {
            if(theme_button.get_active()) {
                plank_button.set_active(false);
                icon_button.set_active(false);
                return true;
            } else {
                return false;
            }
        }

        // if icon is checked
        private bool set_icon_status() {
            if(icon_button.get_active()) {
                plank_button.set_active(false);
                theme_button.set_active(false);
                return true;
            } else {
                return false;
            }
        }

        private bool set_plank_status() {
            if(plank_button.get_active()) {
                theme_button.set_active(false);
                icon_button.set_active(false);
                return true;
            } else {
                return false;
            }
        }

        // file has been recieved
        private void on_drag_data_received(Gdk.DragContext drag_context, int x, int y,
                                           Gtk.SelectionData data, uint info, uint time) {
            bool gen_run_stat = false;
            bool gen_notify = true;
            foreach(string uri in data.get_uris()) {
                if(gen_run_stat == true) { gen_notify = false; }
                file = uri.replace("file://","").replace("file:/","");
                file = Uri.unescape_string(file);

                Gtk.drag_finish(drag_context, true, false, time);

                execute_drag(file, theme, icon, plank, gen_notify);
                gen_run_stat = true;
            }
        }

        public void execute_drag(string file, bool theme, bool icon, bool plank, bool notify) {
            // using sync instead of async waites for exit from script
            // not using begin_show: this is a python task (in this case)
            try {
                GLib.Process.spawn_command_line_sync(@"python $DATADIR/dragdrop.py '''$file''' '$theme' '$icon' '$plank' '$notify' 'true'");
            } catch(SpawnError e) {
                stdout.printf("ERROR: %s\n", e.message);
            }
        }
    }
}
