//* -*- Mode: vala; indent-tabs-mode: nil; tab-width: 4 -*-
/* Copyright (c) 2014 Adam Bieńkowski (http://launchpad.net/power-installer)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authored by: Corentin Noel <tintou@mailoo.org>
 */

// main application

using GLib;
using Gtk;
using Granite.Services;

namespace PowerInstaller {

    private void begin_show() {
        execute_command_async(@"python $DATADIR/notification.py '' ''");
    }

    // resources
    private const string DATADIR = Constants.PKGDATADIR;

    // dont show window when API initialized
    private bool dont_show_window;

    private const string PROGRAM_NAME = "Power Installer";

    // main class (Granite.Application)
    public class PowerInstallerApp : Granite.Application {

        construct {
            program_name = PROGRAM_NAME;
            exec_name = "power-installer";

            build_data_dir = Constants.DATADIR;
            build_pkg_data_dir = Constants.PKGDATADIR;
            build_release_name = Constants.RELEASE_NAME;
            build_version = Constants.VERSION;
            build_version_info = Constants.VERSION_INFO;

            app_years = "2014-2015";
            app_icon = "power-installer";
            app_launcher = "power-installer.desktop";
            application_id = "org.power-installer";

            main_url = "https://launchpad.net/power-installer";
            bug_url = "https://bugs.launchpad.net/power-installer";
            help_url = "https://code.launchpad.net/power-installer";
            translate_url = "https://translations.launchpad.net/power-installer";

            about_authors = { "Adam Bieńkowski <donadigos159@gmail.com>" };
            about_documenters = { "Adam Bieńkowski <donadigos159@gmail.com>" };
            about_artists = { "Adam Bieńkowski <donadigos159@gmail.com>" };
            about_comments = "Power Installer let's you install things
quickly on Ubuntu based distributions.";
            about_translators = "";

            about_license_type = Gtk.License.GPL_3_0;
        }

        public PowerInstallerApp() {
            Logger.initialize("PowerInstaller");
            Logger.DisplayLevel = LogLevel.DEBUG;
        }

        public static int main(string[] args) {
            Gtk.init(ref args);
            dont_show_window = false;

            if(args.length > 0) {
                bool main_run_stat = false;
                bool main_notify = true;

                var api = new API();
                var general_tab = new GeneralTab();

                foreach(string file in args) {
                    if(main_run_stat == true) { main_notify = false; }
                    string? init_api = GLib.Environment.get_variable("INIT_API");
                    // get_variable returns string instead of bool
                    if(init_api == "true") {
                        api.init();
                        dont_show_window = true;
                    }

                    // first argument is always the invocation of the program itself
                    if(file != args[0]) {
                        general_tab.execute_drag(file, false, false, false, main_notify);
                        main_run_stat = true;
                    } else {
                        continue;
                    }
                }
            }

            // run app
            var app = new PowerInstallerApp();
            return app.run(args);
        }

        public override void activate() {
            if(dont_show_window == false) {
                var window = new Window();
                window.destroy.connect(Gtk.main_quit);
                window.show_all();
                Gtk.main();
            }
        }
    }
}
