//* -*- Mode: vala; indent-tabs-mode: nil; tab-width: 4 -*-
/* Copyright (c) 2014 Adam Bieńkowski (http://launchpad.net/power-installer)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authored by: Corentin Noel <tintou@mailoo.org>
 */

// main window

namespace PowerInstaller {

    public class Window : Gtk.Window {
        private Gtk.Stack main_stack;
        private Gtk.StackSwitcher stack_switcher;

        public Window() {
            this.title = PROGRAM_NAME;
            this.window_position = Gtk.WindowPosition.CENTER;
            this.set_resizable(false);
            this.set_keep_above(true);
            this.border_width = 6;
            this.set_default_size(380, 400);
            this.create_widgets();
        }

        private void create_widgets() {
            // main box
            var main_grid = new Gtk.Grid();

            var header = new Gtk.HeaderBar();
            header.set_title(PROGRAM_NAME);
            header.set_show_close_button(true);
            this.set_titlebar(header);

            var mini = new Gtk.Image();
            mini.set_from_file(DATADIR + "/miniature.svg");
			header.pack_end(mini);

            // stack
            main_stack = new Gtk.Stack();
            main_stack.set_hexpand(true);
            main_stack.set_transition_type(Gtk.StackTransitionType.SLIDE_LEFT);

            // stack switcher
            stack_switcher = new Gtk.StackSwitcher();
            stack_switcher.stack = main_stack;
            stack_switcher.halign = Gtk.Align.CENTER;
            stack_switcher.margin_top = 2;

            // add tabs to dialog
            var general_tab = new GeneralTab();
            general_tab.expand = true;

            var command_tab = new CommandTab();
            command_tab.expand = true;

            var actions_tab = new ActionsTab();
            actions_tab.expand = true;

            main_stack.add_titled(general_tab, "general", _("General"));
            main_stack.add_titled(command_tab, "commands", _("Commands"));
            main_stack.add_titled(actions_tab, "actions", _("Actions"));

            // add stack and stack switcher to main grid
            main_grid.add(stack_switcher);
            main_grid.attach(main_stack, 0, 1, 1, 1);
            main_grid.show_all();
            this.add(main_grid);
        }
    }
}
